<?php
/*
Plugin Name: HitCount
Description: Counts page hits and visitors
Version: 0.7
Author: Martin Vlcek
Author URI: http://mvlcek.bplaced.net
*/

# get correct id for plugin
$thisfile = basename(__FILE__, ".php");

define('HITCOUNT_DIR', 'hitcount/'); 

# register plugin
register_plugin(
	$thisfile, 
	'HitCount', 	
	'0.7', 		
	'Martin Vlcek',
	'http://mvlcek.bplaced.net', 
	'Counts page hits and visitors',
	'support',
	'hitcount_overview'  
);

# activate filter
add_action('index-pretemplate', 'hitcount_init');
add_action('support-sidebar', 'createSideMenu', array($thisfile, 'Hits & Visitors'));

function hitcount_init() {
  global $url;
  $hcdir = GSDATAOTHERPATH . HITCOUNT_DIR;
  if (!file_exists($hcdir)) {
    mkdir(substr($hcdir,0,strlen($hcdir)-1), 0777);
    $fp = fopen($hcdir . '.htaccess', 'w');
    fputs($fp, 'Deny from all');
    fclose($fp);
  }
  @session_start();
  $lp = fopen(GSDATAOTHERPATH . HITCOUNT_DIR . 'lock.txt', 'w');
  if (flock($lp, LOCK_EX)) {
    # visitors
    if (!isset($_SESSION['hitcount_visits'])) {
      $_SESSION['hitcount_visits'] = hitcount_count($hcdir . 'visits.txt');
    }
    # hits
    hitcount_count($hcdir . 'hits_' . $url . '.txt');
    # log
    $fp = fopen(GSDATAOTHERPATH . HITCOUNT_DIR . 'log.txt', 'a');
    $referer = @$_SERVER["HTTP_REFERER"];
    $useragent = @$_SERVER["HTTP_USER_AGENT"];
    fputs($fp, time() . " " . $_SESSION['hitcount_visits'] . " " . $url . " " . $referer . " " . $useragent . "\n");
    fclose($fp);
    flock($lp, LOCK_UN);
  }
  fclose($lp);
}

function hitcount_count($hcfile) {
  if (file_exists($hcfile)) {
    $hits = file($hcfile);
    $hits = $hits[0] + 1;
  } else {
    $hits = 1;
  }
  $fp = fopen($hcfile, 'w');
  fputs($fp, $hits);
  fclose($fp);
  return $hits;
}

function hitcount_reset() {
  $dir_handle = @opendir(GSDATAOTHERPATH . HITCOUNT_DIR) or die("Unable to open hitcount directory");
  while ($filename = readdir($dir_handle)) {
    if (!is_dir(GSDATAOTHERPATH . HITCOUNT_DIR . $filename) && $filename != '.htaccess') {
      unlink(GSDATAOTHERPATH . HITCOUNT_DIR . $filename);
    }
  }
}

function hitcount_cmp($a, $b) {
  return $a['hits'] == $b['hits'] ? strcmp($a['url'],$b['url']) : $b['hits'] - $a['hits']; 
}

function hitcount_cmp_ua($a, $b) {
  if (count($a['visitors']) != count($b['visitors'])) {
    return count($b['visitors']) - count($a['visitors']);
  } else if ($a['hits'] != $b['hits']) {
    return $b['hits'] - $a['hits'];
  } else {  
    return strcmp($a['name'],$b['name']);
  }  
}

function hitcount_cmp_ref($a, $b) {
  return $a['visitors'] != $b['visitors'] ? $b['visitors'] - $a['visitors'] : strcmp($a['name'],$b['name']);
}

function hitcount_overview() {
  if (isset($_REQUEST['hitcount_reset'])) hitcount_reset();
  if (isset($_REQUEST['hitcount_time'])) {
    hitcount_show_time();
  } else if (isset($_REQUEST['hitcount_referers'])) {
    hitcount_show_referers();
  } else if (isset($_REQUEST['hitcount_useragents'])) {
    hitcount_show_useragents();
  } else if (isset($_REQUEST['hitcount_duration'])) {
    hitcount_show_duration();
  } else {
    hitcount_show_hitcounts();
  }
}

function hitcount_show_time() {
?>
  <label>Hits &amp; Visitors over Time</label><?php hitcount_show_links('time'); ?>
<?php
}

function hitcount_show_referers() {
  global $SITEURL;
  $referers = array();
  $visitors = array();
  $f = @fopen(GSDATAOTHERPATH . HITCOUNT_DIR . 'log.txt', "r");
  if ($f) {
    while (($line = fgets($f)) !== false) {
      $parts = preg_split('/ /', $line, 5);
      $vi = $parts[1];
      $re = $parts[3];
      if (trim($re) != '' && substr($re,0,strlen($SITEURL)) != $SITEURL && 
            !preg_match('/^https?:\/\/localhost/i',$re) &&!isset($visitors[$vi])) {
        $visitors[$vi] = 1;
        if (!isset($referers[$re])) {
          $referers[$re] = array('name' => $re, 'visitors' => 1);
        } else {
          $referers[$re]['visitors']++;
        }
      }
    }
    fclose($f);
  }
  usort($referers,'hitcount_cmp_ref');
?>
  <label>Referers</label><?php hitcount_show_links('referers'); ?>
  <table id="referers" class="edittable highlight">
    <tr><th>Referer</th><th style="width:10%">Visitors</th></tr>
<?php if (count($referers) > 0) foreach ($referers as $re) { ?>
    <tr><td><?php echo $re['name']; ?></td><td style="width:10%"><?php echo $re['visitors']; ?></td></tr>
<?php } ?>
  </table>
<?php
}

function hitcount_show_useragents() {
  $useragents = array();
  $f = @fopen(GSDATAOTHERPATH . HITCOUNT_DIR . 'log.txt', "r");
  if ($f) {
    while (($line = fgets($f)) !== false) {
      $parts = preg_split('/ /', $line, 5);
      $ua = $parts[4];
      $vi = $parts[1];
      $isRobot = preg_match('/bot|spider|crawler|curl|slurp|^$/i', $ua);
      if (!isset($useragents[$ua])) {
        $useragents[$ua] = array('name' => $ua, 'hits' => 1, 'visitors' => array($vi), 'robot' => $isRobot);
      } else {
        $useragents[$ua]['hits']++;
        if (!in_array($vi, $useragents[$ua]['visitors'])) $useragents[$ua]['visitors'][] = $vi;
      }
    }
    fclose($f);
  }
  usort($useragents,'hitcount_cmp_ua');
?>
  <label>User agents</label><?php hitcount_show_links('useragents'); ?>
  <table id="useragents" class="edittable highlight">
    <tr><th>User agent</th><th style="width:10%">Hits</th><th style="width:10%">Visitors</th></tr>
<?php if (count($useragents) > 0) foreach ($useragents as $ua) { ?>
    <tr><td <?php echo $ua['robot'] ? 'style="color:#888888"' : ''; ?>><?php echo $ua['name']; ?></td><td style="width:10%"><?php echo $ua['hits']; ?></td><td style="width:10%"><?php echo count($ua['visitors']); ?></td></tr>
<?php } ?>
  </table>
<?php
}

function hitcount_show_duration() {
  $durations = array(
    array('max' => 30, 'text' => '0s - 30s', 'num' => 0),
    array('max' => 60, 'text' => '30s - 1min', 'num' => 0),
    array('max' => 120, 'text' => '1min - 2min', 'num' => 0),
    array('max' => 180, 'text' => '2min - 3min', 'num' => 0),
    array('max' => 240, 'text' => '3min - 4min', 'num' => 0),
    array('max' => 300, 'text' => '4min - 5min', 'num' => 0),
    array('max' => 600, 'text' => '5min - 10min', 'num' => 0),
    array('max' => 900, 'text' => '10min - 15min', 'num' => 0),
    array('max' => 1800, 'text' => '15min - 30min', 'num' => 0),
    array('max' => 2700, 'text' => '30min - 45min', 'num' => 0),
    array('max' => 3600, 'text' => '45min - 1h', 'num' => 0),
    array('max' => 0, 'text' => '1h+', 'num' => 0)
  );
  $visits = array();
  $f = @fopen(GSDATAOTHERPATH . HITCOUNT_DIR . 'log.txt', "r");
  if ($f) {
    while (($line = fgets($f)) !== false) {
      $parts = preg_split('/ /', $line, 5);
      $ti = (int) $parts[0];
      $vi = $parts[1];
      if (!isset($visits[$vi])) {
        $visits[$vi] = array('start' => $ti, 'duration' => 0);
      } else {
        $visits[$vi]['duration'] = $ti - $visits[$vi]['start'];
      }
    }
    fclose($f);
  }
  if (count($visits) > 0) {
    foreach ($visits as $visit) {
      $i = 0; while ($durations[$i]['max'] > 0 && $durations[$i]['max'] < $visit['duration']) $i++;
      $durations[$i]['num']++;
    }
  }
?>
  <label>Visit durations</label><?php hitcount_show_links('duration'); ?>
  <table id="useragents" class="edittable highlight">
    <tr><th>Duration</th><th style="width:10%">Visitors</th></tr>
<?php foreach ($durations as $du) { ?>
    <tr><td><?php echo $du['text']; ?></td><td style="width:10%"><?php echo $du['num']; ?></td></tr>
<?php } ?>
  </table>
<?php
}

function hitcount_show_hitcounts() {
  global $i18n;
  $visits = 0;
  $totalhits = 0;
  $hits = array();
  $dir_handle = @opendir(GSDATAOTHERPATH . HITCOUNT_DIR);
  while ($filename = @readdir($dir_handle)) {
    if (!is_dir(GSDATAOTHERPATH . HITCOUNT_DIR . $filename)) {
      if ($filename == 'visits.txt') {
        $lines = file(GSDATAOTHERPATH . HITCOUNT_DIR . $filename);
        $visits = $lines[0];
      } else if (substr($filename, 0, 5) == 'hits_') {
        $url = substr($filename, 5, strlen($filename)-9);
        $lines = file(GSDATAOTHERPATH . HITCOUNT_DIR . $filename);
   			$data = getXML(GSDATAPAGESPATH . $url . '.xml');
        $hits[] = array('url' => $url, 'hits' => 0+$lines[0], 'title' => ''.$data->title, 'date' => $data->pubDate); 
        $totalhits += $lines[0];
      }
    }
  }
  usort($hits,'hitcount_cmp');
?>
  <label>Hits &amp; Visitors</label><?php hitcount_show_links('hitcounts'); ?>
  <table id="hits" class="edittable highlight">
  <tr><td><strong>Visits</strong></td><td style="width:10%"><?php echo $visits; ?></td></tr>
  <tr><td><strong>Total number of hits</strong></td><td style="width:10%"><?php echo $totalhits; ?></td></tr>
<?php if (count($hits) > 0) foreach ($hits as $page) { ?>
  <tr>
<?php if ($page['title']) { ?>
    <td><a title="<?php echo $i18n['EDITPAGE_TITLE'].': '.cl($page['title']).' - '.shtDate($page['date']); ?>" href="edit.php?id=<?php echo $page['url']; ?>"><?php echo cl($page['title']); ?></a></td>
<?php } else { ?>
    <td><?php echo $page['url']; ?></td>
<?php } ?>
    <td style="width:10%"><?php echo $page['hits']; ?></td>
  </tr>
<?php } ?>
  </table>
  <form method="post">
    <input type="hidden" name="id" value="hitcount"/>
    <input type="submit" name="hitcount_reset" value="Reset"/>
  </form>
<?php  
}

function hitcount_show_links($current) {
  $link = "load.php?id=hitcount";
  echo "<div class=\"edit-nav\">";
  if ($current != 'hitcounts') echo "<a href=\"{$link}&hitcount_hitcounts=1\">Hit counts</a> ";
  if (false && $current != 'time') echo "<a href=\"{$link}&hitcount_time=1\">Time</a> ";
  if ($current != 'referers') echo "<a href=\"{$link}&hitcount_referers=1\">Referers</a> ";
  if ($current != 'useragents') echo "<a href=\"{$link}&hitcount_useragents=1\">User agents</a> ";
  if ($current != 'duration') echo "<a href=\"{$link}&hitcount_duration=1\">Duration</a> ";
  echo "<div class=\"clear\"></div></div>";
}

